﻿using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.InteropServices;
using System.Resources;
using System.IO;

/// <summary>
/// Aplicação criada para traduções rápidas
/// Usa o tradutor em http://www.translate.google.com/.  
/// que é um program a open-source,
/// e pode ser modificaado para propósitos que não comerciais
/// de outra forma violará os termos de copyright do Googles
/// </summary>

namespace QuickTranslation
{
    public partial class Main : System.Windows.Forms.Form
    {
        /// <summary>
        /// Importe as DLLs necessária para monitorar o clipboard.  
        /// Sem essas referências a aplicação não funciona
        /// </summary>
        
        [DllImport("User32.dll")]
        protected static extern int SetClipboardViewer(int hWndNewViewer);

        [DllImport("User32.dll", CharSet = CharSet.Auto)]
        public static extern bool ChangeClipboardChain(IntPtr hWndRemove, IntPtr hWndNewNext);

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        public static extern int SendMessage(IntPtr hwnd, int wMsg, IntPtr wParam, IntPtr lParam);

        IntPtr nextClipboardViewer;
        
        // Inicio //

        public Main()
        {
            InitializeComponent();
            nextClipboardViewer = (IntPtr)SetClipboardViewer((int)this.Handle);  
        }
        [STAThread]       
 
        protected override void WndProc(ref System.Windows.Forms.Message m)
        {
            // defined in winuser.h
            const int WM_DRAWCLIPBOARD = 0x308;
            const int WM_CHANGECBCHAIN = 0x030D;

            switch (m.Msg)
            {
                case WM_DRAWCLIPBOARD:
                    DisplayClipboardData();
                    SendMessage(nextClipboardViewer, m.Msg, m.WParam, m.LParam);
                    break;

                case WM_CHANGECBCHAIN:
                    if (m.WParam == nextClipboardViewer)
                        nextClipboardViewer = m.LParam;
                    else
                        SendMessage(nextClipboardViewer, m.Msg, m.WParam, m.LParam);
                    break;

                default:
                    base.WndProc(ref m);
                    break;
            }
        }

        /// <summary>
        /// Controla o painel o texto da barra de tarefas
        /// O texto somente aparece uma vez durante o progarama
        /// </summary>
        /// 
        public static class BuyMeACup
        {
            static int _BuyMeACup;
            public static int BuyMeACupPlease
            {
                get
                {
                    return _BuyMeACup;
                }
                set
                {
                    _BuyMeACup = value;
                }
            }
        }

        /// <summary>
        /// Roltina mais importante
        /// A consulta do tradutor translator.cs
        /// incluido na aplicação. Obtem o valor do clipboard e realiza varias ações
        /// dependendo da seleção do usuário
        /// </summary>

        void DisplayClipboardData()
        {
            try
            {
                // Declare our new iData handler.  This will determine the clipboard contents.
                IDataObject iData = new DataObject();
                iData = Clipboard.GetDataObject();

                // Now we will create a new GoogleTranslator, thank you RavSoft!
                RavSoft.GoogleTranslator.Translator t = new RavSoft.GoogleTranslator.Translator();

                /* If RTF is what lies in the computers clipboard then we will handle that
                 * here on these lines.  The basic construct for a Google query is very 
                 * consistant, and does not change at all. */

                if (iData.GetDataPresent(DataFormats.Rtf))
                {
                    // Set the source language = _comboFrom
                    t.SourceLanguage = (string)this._comboFrom.SelectedItem;
                    // Set the target language = _comboTo
                    t.TargetLanguage = (string)this._comboTo.SelectedItem;
                    // Get our source text (iData.GetData(DataFormats.Text)
                    // We will ignore pictures! Ha Ha!
                    //t.SourceText = (string)iData.GetData(DataFormats.Text);
                    t.SourceText = Clipboard.GetText();
                    // Call the translator.
                    t.Translate();
                    /* Now if they have the translator enabled we will get the return
                     * data and send it out through the keyboard (this comes in very
                     * handy when chatting online & so-forth. */
                    if (conversionEnabled.Checked)
                    {
                        // Send the keys to the keyboard.
                        SendKeys.Send(t.Translation.ToString());
                    }
                    // If Conversion is disabled then now we must handle that.
                    else
                    {
                        /* Here we will check if they just want the data copied to the
                         * clipboard.  If this is off then we will do nothing with the
                         * data, or the clipboard for that matter. */

                        if (justCopy.Checked)
                        {
                            // Notify the user [via] the balloon text.
                            Clipboard.SetText(t.Translation.ToString());
                            sysTray.BalloonTipIcon = ToolTipIcon.Info;
                            sysTray.BalloonTipTitle = (string)_comboFrom.SelectedItem + " Para " + (string)_comboTo.SelectedItem;
                            sysTray.BalloonTipText = Clipboard.GetText() + " Colocado no ClipBoard.";
                            sysTray.ShowBalloonTip(10);
                        }
                    }
                }
                // This is where plain text w/out formatting is handled.
                else if (iData.GetDataPresent(DataFormats.Text))
                {
                    // Set the source language = _comboFrom
                    t.SourceLanguage = (string)this._comboFrom.SelectedItem;
                    // Set the target language = _comboTo
                    t.TargetLanguage = (string)this._comboTo.SelectedItem;
                    // Get our source text (iData.GetData(DataFormats.Text)
                    // We will ignore pictures! Ha Ha!
                  //  t.SourceText = (string)iData.GetData(DataFormats.Text);
                    // Call the translator.
                    t.SourceText = Clipboard.GetText();
                    t.Translate();
                    /* Now if they have the translator enabled we will get the return
                    * data and send it out through the keyboard (this comes in very
                    * handy when chatting online & so-forth. */
                    if (conversionEnabled.Checked)
                    {
                        // Send the keys to the keyboard.
                        SendKeys.Send(t.Translation.ToString());
                    }
                    // If Conversion is disabled then now we must handle that.
                    else
                    {
                        /* Here we will check if they just want the data copied to the
                         * clipboard.  If this is off then we will do nothing with the
                         * data, or the clipboard for that matter. */

                        if (justCopy.Checked)
                        {
                            // avisa o usuário 
                            Clipboard.SetText(t.Translation.ToString());
                            sysTray.BalloonTipIcon = ToolTipIcon.Info;
                            sysTray.BalloonTipTitle = (string)_comboFrom.SelectedItem + " Para " + (string)_comboTo.SelectedItem;
                            sysTray.BalloonTipText = Clipboard.GetText() + " colocado no clipboard.";
                            sysTray.ShowBalloonTip(10);
                        }
                     }
                   }
                else return;
            }
            catch 
            {
             return; // Ignora os erros
            }
        }
        private void Form1_Load(object sender, EventArgs e)
        {
            // esconde o formulário principal e crie icone na barra de tarefas
            this.WindowState = FormWindowState.Minimized;
            this.Visible = false;

            // avisa o usuário
            sysTray.BalloonTipIcon = ToolTipIcon.Info;
            sysTray.BalloonTipTitle = "Tradutor Rápido";
            sysTray.BalloonTipText = "O Tradutor esta em execução. Para usá-lo clique duas vezes neste ícone.";
            sysTray.ShowBalloonTip(10);

            Clipboard.Clear();
            /// <summary>
            /// carrega 5 configurações
            /// </summary>
            
            int ConversionEnabled = Properties.Settings.Default.conversion;
            int ClipboardEnabled = Properties.Settings.Default.clipboard;
            int SavingSettings = Properties.Settings.Default.savesettings;

            String FromLanguage = Properties.Settings.Default.fromlang;
            String ToLanguage = Properties.Settings.Default.tolang;

            if (SavingSettings == 1)
            {
                saveSettings.CheckState = CheckState.Checked;
            }
            else
            {
                saveSettings.CheckState = CheckState.Unchecked;
            }
            if (ConversionEnabled == 1)
            {
                conversionEnabled.CheckState = CheckState.Checked;
                justCopy.CheckState = CheckState.Unchecked;
                offToolStripMenuItem.Enabled = true;
                onToolStripMenuItem.Enabled = false;
                offToolStripMenuItem1.Enabled = false;
            }
            else if (ClipboardEnabled == 1)
            {
                // 
                conversionEnabled.CheckState = CheckState.Unchecked;
                justCopy.CheckState = CheckState.Checked;
                offToolStripMenuItem1.Enabled = true;
                onToolStripMenuItem1.Enabled = false;
                offToolStripMenuItem.Enabled = false;
            }
            else
            {
                conversionEnabled.CheckState = CheckState.Unchecked;
                justCopy.CheckState = CheckState.Unchecked;
                offToolStripMenuItem.Enabled = true;
                onToolStripMenuItem.Enabled = true;
                offToolStripMenuItem1.Enabled = true;
                onToolStripMenuItem1.Enabled = true;
            }

            this._comboFrom.SelectedItem = FromLanguage;
            this._comboTo.SelectedItem = ToLanguage;

        }
        void SendMyKeys()
        {
            SendKeys.Send(Clipboard.GetText());
        }

        private void justCopy_CheckedChanged(object sender, EventArgs e)
        {
            if (justCopy.Checked)
            {
                conversionEnabled.CheckState = CheckState.Unchecked;
                
                offToolStripMenuItem.Enabled = false;
                offToolStripMenuItem1.Enabled = true;
                onToolStripMenuItem.Enabled = true;
                onToolStripMenuItem1.Enabled = false;

                if (saveSettings.Checked)
                {
                    Properties.Settings.Default.conversion = 0;
                    Properties.Settings.Default.clipboard = 1;
                    Properties.Settings.Default.Save();
                }
            }
            else
            {
                if (saveSettings.Checked)
                {
                    Properties.Settings.Default.clipboard = 0;
                    Properties.Settings.Default.Save();
                }
            }
        }

        private void conversionEnabled_CheckedChanged(object sender, EventArgs e)
        {
            if (conversionEnabled.Checked)
            {
                justCopy.CheckState = CheckState.Unchecked;

                offToolStripMenuItem.Enabled = true;
                offToolStripMenuItem1.Enabled = false;
                onToolStripMenuItem.Enabled = false;
                onToolStripMenuItem1.Enabled = true;

                if (saveSettings.Checked)
                {
                    Properties.Settings.Default.conversion = 1;
                    Properties.Settings.Default.clipboard = 0;
                    Properties.Settings.Default.Save();
                }

            }
            else
            {
                if (saveSettings.Checked)
                {
                    Properties.Settings.Default.conversion = 0;
                    Properties.Settings.Default.Save();
                }
            }
        }
        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            About ShowAbout = new About();
            ShowAbout.Show();
        }

        private void onToolStripMenuItem_Click(object sender, EventArgs e)
        {
            conversionEnabled.CheckState = CheckState.Checked;
            onToolStripMenuItem.Enabled = false;
            offToolStripMenuItem.Enabled = true;

            if (saveSettings.Checked)
            {
                Properties.Settings.Default.conversion = 1;
                Properties.Settings.Default.clipboard = 0;
                Properties.Settings.Default.Save();
            }
        }

        private void offToolStripMenuItem_Click(object sender, EventArgs e)
        {
            conversionEnabled.CheckState = CheckState.Unchecked;
            onToolStripMenuItem.Enabled = true;
            offToolStripMenuItem.Enabled = false;

            if (saveSettings.Checked)
            {
                Properties.Settings.Default.conversion = 0;
                Properties.Settings.Default.clipboard = 1;
                Properties.Settings.Default.Save();
            }
        }

        private void _comboFrom_SelectedIndexChanged(object sender, EventArgs e)
        {
            _inputBox.Text = "Informe o texto....";
            _outputBox.Text = "Resultado....";
            if (saveSettings.Checked)
            {
                Properties.Settings.Default.fromlang = (string)this._comboFrom.SelectedItem;
                Properties.Settings.Default.Save();
            }
        }

        private void _comboTo_SelectedIndexChanged(object sender, EventArgs e)
        {
            _inputBox.Text = "Informe o texto....";
            _outputBox.Text = "Resultado...";
            if (saveSettings.Checked)
            {
                Properties.Settings.Default.tolang = (string)this._comboTo.SelectedItem;
                Properties.Settings.Default.Save();
            }
        }

        private void saveSettings_CheckedChanged(object sender, EventArgs e)
        {
            if (saveSettings.Checked)
            {
                Properties.Settings.Default.savesettings = 1;
                Properties.Settings.Default.Save();
            }
            else
            {
                Properties.Settings.Default.savesettings = 0;
                Properties.Settings.Default.Save();
            }
        }

        private void onToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            justCopy.CheckState = CheckState.Checked;
            onToolStripMenuItem1.Enabled = false;
            offToolStripMenuItem1.Enabled = true;

            if (saveSettings.Checked)
            {
                Properties.Settings.Default.conversion = 0;
                Properties.Settings.Default.clipboard = 1;
                Properties.Settings.Default.Save();
            }
        }

        private void offToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            justCopy.CheckState = CheckState.Unchecked;
            onToolStripMenuItem1.Enabled = true;
            offToolStripMenuItem1.Enabled = false;

            if (saveSettings.Checked)
            {
                Properties.Settings.Default.conversion = 1;
                Properties.Settings.Default.clipboard = 0;
                Properties.Settings.Default.Save();
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            if (FormWindowState.Minimized == WindowState)
            {
                this.Hide();
            }
        }

        private void sysTray_DoubleClick(object sender, EventArgs e)
        {
            this.Show();
            WindowState = FormWindowState.Normal;
        }



        private void switchLanguage_MouseEnter(object sender, EventArgs e)
        {
            switchLanguage.BackgroundImage = Properties.Resources.smouse;
        }

        private void switchLanguage_MouseLeave(object sender, EventArgs e)
        {
            switchLanguage.BackgroundImage = Properties.Resources._switch;
        }

        private void switchLanguage_MouseDown(object sender, MouseEventArgs e)
        {
            switchLanguage.BackgroundImage = Properties.Resources.spress;
            
        }

        private void switchLanguage_MouseUp(object sender, MouseEventArgs e)
        {
            string oldFrom = (string)_comboFrom.SelectedItem;
            string oldTo = (string)_comboTo.SelectedItem;

            _comboTo.SelectedItem = oldFrom;
            _comboFrom.SelectedItem = oldTo;
            switchLanguage.BackgroundImage = Properties.Resources.smouse;
        }

        private void exitToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            sysTray.Visible = false;
            this.Close();
        }

        private void exitToolStripMenuItem_Click_1(object sender, EventArgs e)
        {
            sysTray.Visible = false;
            this.Close();
        }

        private void legalDisclaimerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Obrigado por usar este aplicaitvo.  O objetivo e facilitar a vida " +
                            "das pessoas para acessar os serviços de tradução. O aplicativo foi " +
                            "criado usando várias API´s open-source como as ferramentas de idoma do Google. " +
                            " Autor original : ronharsh@live.com", "Aviso", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void buyMeACupToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //chamada ao paypal para doações foi desabilitada
            //System.Diagnostics.Process.Start("https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=BTLLRG5JSDJYC");
        }

        private void buyMeACupToolStripMenuItem_MouseHover(object sender, EventArgs e)
        {
            
            if (BuyMeACup.BuyMeACupPlease == 0)
            {
                sysTray.BalloonTipIcon = ToolTipIcon.Info;
                sysTray.BalloonTipTitle = "Valeu por usar o tradutor!";
                sysTray.BalloonTipText = "Tradutor Rápido , te ajuda a entender o mundo.";
                sysTray.ShowBalloonTip(30);
                BuyMeACup.BuyMeACupPlease++;
            }
            }

        private void _inputBox_TextChanged(object sender, EventArgs e)
        {
            RavSoft.GoogleTranslator.Translator t = new RavSoft.GoogleTranslator.Translator();
            // selecione o idioma de origem
            t.SourceLanguage = (string)this._comboFrom.SelectedItem;
            // define o idioma de destino
            t.TargetLanguage = (string)this._comboTo.SelectedItem;
            // Get our source text (iData.GetData(DataFormats.Text)
            // We will ignore pictures! Ha Ha!
            t.SourceText = _inputBox.Text;
            // Chama o tradutor
            t.Translate();
            _outputBox.Text = null;
            _outputBox.Text = t.Translation.ToString();

        }

        private void _inputBox_Click(object sender, EventArgs e)
        {
            _inputBox.Text = null;
        }
        void refreshData()
        {
            int ConversionEnabled = Properties.Settings.Default.conversion;
            int ClipboardEnabled = Properties.Settings.Default.clipboard;
            int SavingSettings = Properties.Settings.Default.savesettings;

            String FromLanguage = Properties.Settings.Default.fromlang;
            String ToLanguage = Properties.Settings.Default.tolang;

            // verifia se a opção salvar configurações esta marcada
            if (SavingSettings == 1)
            {
                // marca a opção
                saveSettings.CheckState = CheckState.Checked;
            }
            else
            {
                // deixa desmarcada
                saveSettings.CheckState = CheckState.Unchecked;
            }
            // verifica se o tradutor esta habilitado
            if (ConversionEnabled == 1)
            {
                // verifica se há texto para traduzir
                conversionEnabled.CheckState = CheckState.Checked;
                // desmarca
                justCopy.CheckState = CheckState.Unchecked;
                // Desabilita o tradutor no menu de Contexto
                offToolStripMenuItem.Enabled = true;
                // Desabilita o tradutor
                onToolStripMenuItem.Enabled = false;
                // Desabilita o Clipbard 
                offToolStripMenuItem1.Enabled = false;
            }
            else if (ClipboardEnabled == 1)
            {
                // 
                conversionEnabled.CheckState = CheckState.Unchecked;
                justCopy.CheckState = CheckState.Checked;
                offToolStripMenuItem1.Enabled = true;
                onToolStripMenuItem1.Enabled = false;
                offToolStripMenuItem.Enabled = false;
            }
            else
            {
                conversionEnabled.CheckState = CheckState.Unchecked;
                justCopy.CheckState = CheckState.Unchecked;
                offToolStripMenuItem.Enabled = true;
                onToolStripMenuItem.Enabled = true;
                offToolStripMenuItem1.Enabled = true;
                onToolStripMenuItem1.Enabled = true;
            }

            this._comboFrom.SelectedItem = FromLanguage;
            this._comboTo.SelectedItem = ToLanguage;

        }

        private void button1_Click(object sender, EventArgs e)
        {
            saveSettings.CheckState = CheckState.Unchecked;
            conversionEnabled.CheckState = CheckState.Unchecked;
            justCopy.CheckState = CheckState.Unchecked;

            if (_outputBox.Text == string.Empty)
            {
                MessageBox.Show("Informe um texto para tradução.");
               return;
            }
            Clipboard.SetText(_outputBox.Text);
            refreshData();
            saveSettings.CheckState = CheckState.Checked;
            DisplayClipboardData();
        }

        private void Main_FormClosed(object sender, FormClosedEventArgs e)
        {
            sysTray.Visible = false;
        }
        }
    }


